package com.jzdsh.common;

import org.apache.commons.lang3.StringUtils;

import java.awt.geom.Line2D;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.util.*;

/**
 * @Author FengZhen
 * @Date 2019-12-02 16:01
 * @Description 系统采样
 * 粗采样的算法
 * 输入：
 * 开始位置， 结束位置， 步长（采样率）， 是否开启位数不足补0， 单号的位数，前缀
 * start_index, end_index, step, flag, id_num，‘888’
 *
 * 输出： <k,v>
 * K：小号段（start_end）
 * V:实际的单号
 */
public class SystemRandomUtil {

    /**
     * 系统采样
     * @param startIndex    开始位置
     * @param endIndex      结束位置
     * @param stepLength    步长
     * @param billLength    单号的位数
     * @param prefix        前缀
     * @return              MAP<小号段,实际单号>
     */
    public static Map<String, String> getSystemRandom(Long startIndex, Long endIndex, Long stepLength, int billLength, String prefix) throws Exception {
        SecureRandom random= SecureRandom.getInstance("SHA1PRNG");
        Map<String, String> resultMap = new HashMap<>();
        if (
                null == startIndex
                || null == endIndex
                || endIndex < startIndex
                || stepLength > (endIndex - startIndex)
        ){
            //过滤异常
            throw new Exception("incorrect parameters");
        }
        //获取总长
        Long maxLength = endIndex - startIndex;
        //最后一组元素个数
        Long lastGroupSize = maxLength % stepLength;
        //组数
        Long groupSize = lastGroupSize > 0 ? maxLength / stepLength + 1 : maxLength / stepLength;
        //除去前缀的位数
        int numLength = billLength - (StringUtils.isBlank(prefix) ? 0 : prefix.length());


        Map<Long,Long> randomMap = new HashMap<>();
        for (Long i = 0L; i < groupSize; i++){
            randomMap.put(i,getLongRandom(stepLength + 1));
        }

        //每组获取随机数
        for (Long i = 0L; i < groupSize; i++){
            //组内的开始位
            Long groupStartIndex = startIndex + stepLength * i;
            //处理最后一组
            if (i.equals(groupSize - 1) && lastGroupSize > 0){
                stepLength = lastGroupSize;
            }
            //组内的结束位
            Long groupEndIndex = groupStartIndex + stepLength;
            //获取随机数
            //Long randomNum = getLongRandom(stepLength + 1);
            Long randomNum = randomMap.get(i);
            //生成抽样单号 每组开始位 + 随机数
            Long resultRandomNum = groupStartIndex + randomNum;
            //抽样单号拼接前缀
            String resultRandomNumStr = StringUtils.isBlank(prefix) ? fillZero(resultRandomNum, numLength) : prefix + fillZero(resultRandomNum, numLength);
            //入map
            //小号段start 拼接前缀、高位补0
            String codeStartIndex = StringUtils.isBlank(prefix) ? fillZero(groupStartIndex, numLength) : prefix + fillZero(groupStartIndex, numLength);
            //小号段end 拼接前缀、高位补0
            String codeEndIndex = StringUtils.isBlank(prefix) ? fillZero(groupEndIndex, numLength) : prefix + fillZero(groupEndIndex, numLength);
            resultMap.put(codeStartIndex + "_" + codeEndIndex, resultRandomNumStr);
        }
        return resultMap;
    }

    /**
     * 指定范围内生成随机数
     * @param size  size以内
     * @return [0,size)
     */
    private static Long getLongRandom(Long size) throws NoSuchAlgorithmException {
        Long s = Math.abs(SecureRandom.getInstance("SHA1PRNG").nextLong() % size);
        return s;
    }

    /**
     * 高位补零
     * @param value     待补位数字
     * @param size      总位数
     * @return
     */
    private static String fillZero(Long value, int size){
        return String.format("%0" + size + "d", value);
    }



    public static void main(String[] args) throws NoSuchAlgorithmException, NoSuchProviderException {
        Map<Long,Integer> aa = new HashMap<>();
        for (int i = 0;i<100000;i++){
            Long randNum = getLongRandom(100L);

            Integer value = aa.get(randNum);
            if (value == null) {
                value = 0;
            }
            value = value + 1;
            aa.put(randNum, value);
        }

        System.out.println(aa);

        List<Long> keyList = new ArrayList<>();
        keyList.addAll(aa.keySet());
        Collections.sort(keyList);

        List<Integer> valueList = new ArrayList<>();

        for (Long bill:keyList){
            //keyList.add(bill);
            valueList.add(aa.get(bill));

        }

        System.out.println(keyList);
        System.out.println(valueList);


    }
}
